"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from copy import deepcopy
from typing import Optional
from typing import TYPE_CHECKING
from typing import Type

from encommon.types import DictStrAny
from encommon.types import NCNone
from encommon.types import clsname
from encommon.utils import array_ansi
from encommon.utils import print_ansi

from .addons import RobieLogger
from .childs import RobieChilds
from .models import RobieModels

if TYPE_CHECKING:
    from .common import RobiePrint
    from .config import RobieConfig
    from .childs import RobieClient
    from .childs import RobiePlugin
    from .params import RobieParams



class Robie:
    """
    Interact with chat networks and integrate using plugins.

    :param config: Primary class instance for configuration.
    """

    __config: 'RobieConfig'

    __logger: RobieLogger

    __childs: RobieChilds


    def __init__(
        self,
        config: 'RobieConfig',
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        config.logger.log_d(
            base=clsname(self),
            status='initial')

        self.__config = config

        self.__logger = (
            RobieLogger(self))

        self.__childs = (
            RobieChilds(self))

        self.childs.validate()

        config.logger.log_d(
            base=clsname(self),
            status='created')


    @property
    def config(
        self,
    ) -> 'RobieConfig':
        """
        Return the Config instance containing the configuration.

        :returns: Config instance containing the configuration.
        """

        return self.__config


    @property
    def logger(
        self,
    ) -> RobieLogger:
        """
        Return the value for the attribute from class instance.

        :returns: Value for the attribute from class instance.
        """

        return self.__logger


    @property
    def childs(
        self,
    ) -> RobieChilds:
        """
        Return the value for the attribute from class instance.

        :returns: Value for the attribute from class instance.
        """

        return self.__childs


    @property
    def params(
        self,
    ) -> 'RobieParams':
        """
        Return the Pydantic model containing the configuration.

        :returns: Pydantic model containing the configuration.
        """

        return self.config.params


    @property
    def dumped(
        self,
    ) -> DictStrAny:
        """
        Return the facts about the attributes from the instance.

        :returns: Facts about the attributes from the instance.
        """

        params = deepcopy(
            self.params.endumped)

        childs = deepcopy(
            self.childs.dumped)

        items = childs.items()

        for key, value in items:
            params[key] = value

        return params


    def printer(
        self,
        source: 'RobiePrint',
        color: int = 6,
    ) -> None:
        """
        Print the contents for the object within Robie instance.

        :param source: Content which will be shown after header.
        :param color: Override the color used for box character.
        """

        params = self.params
        printer = params.printer

        pmessage = printer.message
        pcommand = printer.command


        RobieMessage = (
            RobieModels
            .message())

        RobieCommand = (
            RobieModels
            .command())


        message = isinstance(
            source, RobieMessage)

        if message and not pmessage:
            return NCNone


        command = isinstance(
            source, RobieCommand)

        if command and not pcommand:
            return NCNone


        line: str = '━'

        print_ansi(
            f'\n<c9{color}>┍'
            f'{line * 63}<c0>')

        dumped = array_ansi(
            source, indent=2)

        print(  # noqa: T201
            f'\n{dumped}')

        print_ansi(
            f'\n<c9{color}>┕'
            f'{line * 63}<c0>\n')


    def register(
        self,
        name: str,
        *,
        client: Optional[Type['RobieClient']] = None,
        plugin: Optional[Type['RobiePlugin']] = None,
    ) -> None:
        """
        Register the plugin with the internal operation routine.

        :param name: Name of the object within the Robie config.
        :param client: Class definition for the instantiation.
        :param plugin: Class definition for the instantiation.
        """

        self.childs.register(
            name=name,
            client=client,
            plugin=plugin)



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import TYPE_CHECKING

from encommon.times import Timer
from encommon.types import NCNone

from enconnect.irc import ClientEvent

from .params import AutoJoinPluginParams
from ..status import StatusPlugin
from ..status import StatusPluginStates
from ...robie.childs import RobiePlugin

if TYPE_CHECKING:
    from ...clients import IRCClient
    from ...robie.threads import RobieThread



_CHANNELS = dict[str, set[str]]



class AutoJoinPlugin(RobiePlugin):
    """
    Integrate with the Robie routine and perform operations.

    .. note::
       This plugin maintains joined for configured channels.
    """

    __should: _CHANNELS
    __joined: _CHANNELS

    __timer: Timer


    def __post__(
        self,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        params = self.params

        assert isinstance(
            params,
            AutoJoinPluginParams)

        channels = params.channels

        assert isinstance(
            params,
            AutoJoinPluginParams)

        self.__joined = {}


        should: _CHANNELS = {}
        joined: _CHANNELS = {}

        for channel in channels:

            name = channel.client

            if name not in should:
                should[name] = set()
                joined[name] = set()

            should[name].add(
                channel.channel)

        self.__should = should
        self.__joined = joined


        self.__timer = Timer(5)

        self.__status('pending')


    def validate(
        self,
    ) -> None:
        """
        Perform advanced validation on the parameters provided.
        """


    def operate(
        self,
        thread: 'RobieThread',
    ) -> None:
        """
        Perform the operation related to Homie service threads.

        :param thread: Child class instance for Chatting Robie.
        """

        from ...clients import IRCClient

        should = self.__should
        joined = self.__joined
        robie = self.robie
        childs = robie.childs
        clients = childs.clients
        mqueue = thread.mqueue
        member = thread.member
        cqueue = member.cqueue
        timer = self.__timer


        if timer.ready():
            self.__operate(thread)


        def _autojoin() -> None:

            assert isinstance(
                client, IRCClient)

            _should = should[name]
            _joined = joined[name]

            for join in _should:

                if join in _joined:
                    continue  # NOCVR

                rawcmd = f'JOIN :{join}'

                client.put_command(
                    cqueue, rawcmd)


        while not mqueue.empty:

            mitem = mqueue.get()

            name = mitem.client
            family = mitem.family

            if family != 'irc':
                continue

            client = clients[name]

            assert isinstance(
                client, IRCClient)

            event = getattr(
                mitem, 'event')

            assert isinstance(
                event, ClientEvent)

            self.__process(
                client, event)

            if event.command == '376':
                _autojoin()


    def __operate(
        self,
        thread: 'RobieThread',
    ) -> None:
        """
        Perform the operation related to Homie service threads.

        :param thread: Child class instance for Chatting Robie.
        """

        from ...clients import IRCClient

        should = self.__should
        joined = self.__joined
        robie = self.robie
        childs = robie.childs
        clients = childs.clients
        member = thread.member
        cqueue = member.cqueue

        failure: set[bool] = set()

        self.__status(
            'failure'
            if joined != should
            else 'normal')


        def _autojoin() -> None:

            client = clients[name]

            assert isinstance(
                client, IRCClient)

            _should = should[name]
            _joined = joined[name]

            for join in _should:

                if join in _joined:
                    continue  # NOCVR

                failure.add(True)

                rawcmd = f'JOIN :{join}'

                client.put_command(
                    cqueue, rawcmd)


        for name in should:
            _autojoin()


        self.__status(
            'failure'
            if any(failure)
            else 'normal')


    def __process(
        self,
        client: 'IRCClient',
        event: ClientEvent,
    ) -> None:
        """
        Process the provided message item from the Robie thread.

        :param client: Client class instance for Chatting Robie.
        :param event: Raw event received from the network peer.
        """

        joined = self.__joined
        name = client.name

        if name not in joined:
            return NCNone

        _joined = joined[name]


        isme = event.isme
        command = event.command
        params = event.params

        if params is None:
            return NCNone

        split = params.split(' ')

        current = (
            client.client
            .nickname)


        if (command == 'JOIN'
                and isme is True):

            channel = split[0][1:]

            _joined.add(channel)


        if command == 'KICK':

            channel = split[0]
            target = split[1]

            if target == current:
                _joined.remove(channel)


        if (command == 'PART'
                and isme is True):

            channel = split[0]

            _joined.remove(channel)


    def __status(
        self,
        status: StatusPluginStates,
    ) -> None:
        """
        Update or insert the status of the Robie child instance.

        :param status: One of several possible value for status.
        """

        robie = self.robie
        childs = robie.childs
        plugins = childs.plugins
        params = self.params

        assert isinstance(
            params,
            AutoJoinPluginParams)

        if 'status' not in plugins:
            return None

        plugin = plugins['status']

        assert isinstance(
            plugin, StatusPlugin)

        (plugin.update(
            unique=self.name,
            group='Management',
            title='IRC',
            icon=params.status,
            state=status))



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import TYPE_CHECKING

from encommon.times import Timer

from .params import AutoNickPluginParams
from ..status import StatusPlugin
from ..status import StatusPluginStates
from ...robie.childs import RobiePlugin

if TYPE_CHECKING:
    from ...robie.threads import RobieThread



class AutoNickPlugin(RobiePlugin):
    """
    Integrate with the Robie routine and perform operations.

    .. note::
       This plugin maintains configured nickname on server.
    """

    __timer: Timer


    def __post__(
        self,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        self.__timer = Timer(5)

        self.__status('pending')


    def validate(
        self,
    ) -> None:
        """
        Perform advanced validation on the parameters provided.
        """


    def operate(
        self,
        thread: 'RobieThread',
    ) -> None:
        """
        Perform the operation related to Homie service threads.

        :param thread: Child class instance for Chatting Robie.
        """

        mqueue = thread.mqueue
        timer = self.__timer


        if timer.ready():
            self.__operate(thread)


        while not mqueue.empty:
            mqueue.get()


    def __operate(
        self,
        thread: 'RobieThread',
    ) -> None:
        """
        Perform the operation related to Homie service threads.

        :param thread: Child class instance for Chatting Robie.
        """

        from ...clients import IRCClient
        from ...clients import IRCClientParams

        robie = self.robie
        childs = robie.childs
        member = thread.member
        cqueue = member.cqueue
        params = self.params

        clients = (
            childs.clients
            .values())

        assert isinstance(
            params,
            AutoNickPluginParams)

        names = params.clients

        failure: set[bool] = set()


        def _autonick() -> None:

            assert isinstance(
                client, IRCClient)

            params = client.params

            assert isinstance(
                params,
                IRCClientParams)

            should = (
                params.client
                .nickname)

            current = (
                client.client
                .nickname)

            if current == should:
                return None

            failure.add(True)

            rawcmd = f'NICK :{should}'

            client.put_command(
                cqueue, rawcmd)


        for client in clients:

            name = client.name
            family = client.family

            if family != 'irc':
                continue

            if name in names:
                _autonick()


        self.__status(
            'failure'
            if any(failure)
            else 'normal')



    def __status(
        self,
        status: StatusPluginStates,
    ) -> None:
        """
        Update or insert the status of the Robie child instance.

        :param status: One of several possible value for status.
        """

        robie = self.robie
        childs = robie.childs
        plugins = childs.plugins
        params = self.params

        assert isinstance(
            params,
            AutoNickPluginParams)

        if 'status' not in plugins:
            return None

        plugin = plugins['status']

        assert isinstance(
            plugin, StatusPlugin)

        (plugin.update(
            unique=self.name,
            group='Management',
            title='IRC',
            icon=params.status,
            state=status))



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from threading import Thread
from typing import TYPE_CHECKING

from encommon.types import inrepr
from encommon.types import instr
from encommon.types import lattrs

from ..thread import DupliThread

if TYPE_CHECKING:
    from ...robie import Robie



def test_DupliThread() -> None:
    """
    Perform various tests associated with relevant routines.
    """

    raises = DupliThread(
        thread='invalid')


    attrs = lattrs(raises)

    assert attrs == [
        'thread',
        'about']


    assert inrepr(
        'DupliThread',
        raises)

    assert isinstance(
        hash(raises), int)

    assert instr(
        'Thread (invalid)',
        raises)


    assert str(raises) == (
        'Thread (invalid) '
        'would be duplicate')



def test_DupliThread_cover(
    robie: 'Robie',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param robie: Primary class instance for Chatting Robie.
    """

    thread = Thread(
        name='client')


    raises = DupliThread(
        thread=thread,
        about='about')

    assert str(raises) == (
        'Thread (client) '
        'would be duplicate'
        ' (about)')



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from .child import RobieChild
from .client import RobieClient
from .plugin import RobiePlugin
from .robie import RobieChilds



__all__ = [
    'RobieChild',
    'RobieChilds',
    'RobieClient',
    'RobiePlugin']



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from dataclasses import dataclass
from typing import Literal
from typing import TYPE_CHECKING

from enconnect.discord import ClientEvent

from ...robie.models import RobieMessage

if TYPE_CHECKING:
    from ...robie import Robie
    from ...robie.childs import RobieClient
    from ...robie.models import RobieCommand
    from ...robie.models import RobieMessageKinds



@dataclass
class DSCMessage(RobieMessage):
    """
    Contain information for sharing using the Python queue.
    """

    event: ClientEvent


    def __init__(
        self,
        client: 'RobieClient',
        event: ClientEvent,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        self.event = event

        super().__init__(client=client)


    @property
    def family(
        self,
    ) -> Literal['discord']:
        """
        Return the value for the attribute from class instance.

        :returns: Value for the attribute from class instance.
        """

        return 'discord'


    @property
    def kind(
        self,
    ) -> 'RobieMessageKinds':
        """
        Return the value for the attribute from class instance.

        :returns: Value for the attribute from class instance.
        """

        return self.event.kind


    @property
    def isme(
        self,
    ) -> bool:
        """
        Return the boolean indicating message origin from client.

        :returns: Boolean indicating message origin from client.
        """

        return self.event.isme


    def reply(
        self,
        robie: 'Robie',
        content: str,
    ) -> 'RobieCommand':
        """
        Compose the message and transmit using the chat client.

        :param robie: Primary class instance for Chatting Robie.
        :param content: Content that will be source of message.
        """

        childs = robie.childs
        clients = childs.clients

        client = clients[
            self.client]

        event = self.event


        target = event.recipient

        assert target is not None


        return client.compose(
            target[1], content)



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from .child import InvalidChild
from .param import InvalidParam
from .thread import DupliThread



__all__ = [
    'InvalidChild',
    'InvalidParam',
    'DupliThread']



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import TYPE_CHECKING

from encommon.types import inlist
from encommon.types import inrepr
from encommon.types import instr
from encommon.types import lattrs

if TYPE_CHECKING:
    from ...service import RobieService



def test_RobieClientThread(
    service: 'RobieService',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param service: Ancilary Chatting Robie class instance.
    """

    member = service.clients

    threads = (
        member.threads
        .values())


    for thread in threads:


        attrs = lattrs(thread)

        # Inherits Thread class

        assert inlist(
            '_RobieThread__member',
            attrs)

        assert inlist(
            '_RobieThread__child',
            attrs)

        assert inlist(
            '_RobieThread__mqueue',
            attrs)

        assert inlist(
            '_RobieThread__cqueue',
            attrs)


        assert inrepr(
            'Thread(RobieClient',
            thread)

        assert isinstance(
            hash(thread), int)

        assert instr(
            'Thread(RobieClient',
            thread)


        assert thread.robie

        assert thread.member

        assert thread.child

        assert thread.mqueue

        assert thread.cqueue

        assert not thread.congest

        assert not thread.enqueue



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Literal
from typing import Optional
from typing import TYPE_CHECKING
from typing import Union

if TYPE_CHECKING:
    from ..robie.childs import RobieChild



_PHASE = Literal[
    'initial',
    'runtime']



class InvalidChild(Exception):
    """
    Exception for when the child could not be instantiated.

    .. note::
       Similar or identical to classes in other projects.

    :param child: Name or child that is determined invalid.
    :param phase: From which phase child was found invalid.
    :param about: Additional information for the exception.
    """

    child: str
    phase: _PHASE
    about: Optional[str] = None


    def __init__(
        self,
        child: Union[str, 'RobieChild'],
        phase: _PHASE,
        about: Optional[str] = None,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        if hasattr(child, 'name'):
            child = child.name

        message = (
            f'Child ({child}) '
            'invalid within '
            f'phase ({phase})')

        if about is not None:
            message += (
                f' ({about})')

        self.child = child
        self.about = about

        super().__init__(message)



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from .discord import DSCClient
from .discord import DSCClientParams
from .irc import IRCClient
from .irc import IRCClientParams
from .mattermost import MTMClient
from .mattermost import MTMClientParams



__all__ = [
    'IRCClient',
    'IRCClientParams',
    'DSCClient',
    'DSCClientParams',
    'MTMClient',
    'MTMClientParams']



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from typing import Annotated

from pydantic import Field

from .common import RobieParamsModel



class RobieServiceRespiteParams(RobieParamsModel, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    health: Annotated[
        int,
        Field(3,
              description='How often health is checked',
              ge=1, le=15)]



class RobieServiceParams(RobieParamsModel, extra='forbid'):
    """
    Process and validate the Robie configuration parameters.
    """

    respite: Annotated[
        RobieServiceRespiteParams,
        Field(default_factory=RobieServiceRespiteParams,
              description='When operates are performed')]



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from threading import Thread
from time import sleep as block_sleep
from typing import TYPE_CHECKING

from encommon.types import inrepr
from encommon.types import instr
from encommon.types import lattrs

from enconnect.fixtures import DSCClientSocket
from enconnect.fixtures import IRCClientSocket
from enconnect.fixtures import MTMClientSocket

from ..models import RobieMessage

if TYPE_CHECKING:
    from ..service import RobieService



def test_RobieService(
    service: 'RobieService',
    client_dscsock: DSCClientSocket,
    client_ircsock: IRCClientSocket,
    client_mtmsock: MTMClientSocket,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param service: Ancilary Chatting Robie class instance.
    :param client_ircsock: Object to mock client connection.
    :param client_dscsock: Object to mock client connection.
    :param client_mtmsock: Object to mock client connection.
    """


    attrs = lattrs(service)

    assert attrs == [
        '_RobieService__robie',
        '_RobieService__clients',
        '_RobieService__plugins',
        '_RobieService__timer',
        '_RobieService__vacate',
        '_RobieService__cancel',
        '_RobieService__started']


    assert inrepr(
        'service.RobieService',
        service)

    assert isinstance(
        hash(service), int)

    assert instr(
        'service.RobieService',
        service)


    assert service.robie

    assert service.params

    assert service.clients

    assert service.plugins

    assert len(service.running) == 0

    assert len(service.zombies) == 6


    client_dscsock()
    client_ircsock()
    client_mtmsock()

    service.start()


    thread = Thread(
        target=service.operate)

    thread.start()


    block_sleep(10)

    assert service.running

    assert not service.zombies

    assert not service.congest

    service.soft()

    while service.running:
        block_sleep(1)

    service.stop()

    thread.join()

    assert service.zombies

    assert not service.congest

    assert service.enqueue



def test_RobieService_healths(
    service: 'RobieService',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param service: Ancilary Chatting Robie class instance.
    """

    robie = service.robie
    childs = robie.childs
    clients = childs.clients
    member = service.clients
    threads = member.threads

    client = clients['ircbot']
    thread = threads['ircbot']
    mqueue = thread.mqueue

    model = RobieMessage

    item = model(client)


    for _ in range(6):
        mqueue.put(item)


    assert service.congest

    service.operate_healths()



def test_RobieService_cover(
    service: 'RobieService',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param service: Ancilary Chatting Robie class instance.
    """

    service.stop()
    service.soft()
    service.start()
    service.start()
    service.soft()
    service.soft()
    service.stop()
    service.stop()



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from copy import deepcopy
from threading import Thread
from time import sleep as block_sleep
from typing import TYPE_CHECKING

from encommon.types import inrepr
from encommon.types import instr
from encommon.types import lattrs

from enconnect.fixtures import IRCClientSocket
from enconnect.irc.test import EVENTS as IRCEVENTS

from ..plugin import AutoNickPlugin

if TYPE_CHECKING:
    from ....robie import Robie
    from ....robie import RobieService



IRCEVENTS = deepcopy(IRCEVENTS)



def test_AutoNickPlugin(
    robie: 'Robie',
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param robie: Primary class instance for Chatting Robie.
    """

    childs = robie.childs
    plugins = childs.plugins


    plugin = plugins['autonick']

    assert isinstance(
        plugin, AutoNickPlugin)


    attrs = lattrs(plugin)

    assert attrs == [
        '_RobieChild__robie',
        '_RobieChild__name',
        '_RobieChild__params',
        '_AutoNickPlugin__timer']


    assert inrepr(
        'plugin.AutoNickPlugin',
        plugin)

    assert isinstance(
        hash(plugin), int)

    assert instr(
        'plugin.AutoNickPlugin',
        plugin)


    plugin.validate()

    assert plugin.robie

    assert plugin.enable

    assert plugin.name == 'autonick'

    assert plugin.kind == 'plugin'

    assert plugin.params

    assert plugin.dumped



def test_AutoNickPlugin_cover(
    service: 'RobieService',
    client_ircsock: IRCClientSocket,
) -> None:
    """
    Perform various tests associated with relevant routines.

    :param service: Ancilary Chatting Robie class instance.
    :param client_ircsock: Object to mock client connection.
    """

    client_ircsock(IRCEVENTS)

    service.start()


    thread = Thread(
        target=service.operate)

    thread.start()


    block_sleep(6)

    service.soft()

    while service.running:
        block_sleep(1)

    service.stop()

    thread.join()



"""
Functions and routines associated with Enasis Network Chatting Robie.

This file is part of Enasis Network software eco-system. Distribution
is permitted, for more information consult the project license file.
"""



from threading import Thread
from typing import Optional



class DupliThread(Exception):
    """
    Exception for when the client thread is already running.

    :param thread: Name or thread that would be duplicative.
    :param about: Additional information for the exception.
    """

    thread: str
    about: Optional[str] = None


    def __init__(
        self,
        thread: str | Thread,
        about: Optional[str] = None,
    ) -> None:
        """
        Initialize instance for class using provided parameters.
        """

        if hasattr(thread, 'name'):
            thread = thread.name

        message = (
            f'Thread ({thread}) '
            'would be duplicate')

        if about is not None:
            message += (
                f' ({about})')

        self.thread = thread
        self.about = about

        super().__init__(message)
